@@ -34,7 +34,7 @@ has 'name' => (
 
 has 'capital' => (
   is        => 'ro',
-  isa       => 'Str',
+  isa       => 'Maybe[Str]',
   required  => 1,
 );
 
